import sys
from copy import copy
from functools import partial

import numpy as np
import pytest

bm_limit = 50
bm_count = copy(bm_limit)
hscroll_dir = True
vscroll_dir = True
h_last_time = None
v_last_time = None


# Skip speed-test for CIs
@pytest.mark.skip
@pytest.mark.parametrize('benchmark_param', [{'use_opengl': False},
                                             {'use_opengl': True},
                                             {'precompute': False},
                                             {'precompute': True}])
def test_scroll_speed(raw_orig, benchmark_param, browser_backend):
    """Test the speed of a parameter."""
    # Remove spaces and get params with values
    from time import perf_counter

    from PyQt5.QtCore import QTimer
    from PyQt5.QtWidgets import QApplication

    hscroll_diffs = list()
    vscroll_diffs = list()

    def _ininite_hscroll(pg_fig):
        global bm_count
        global hscroll_dir
        global vscroll_dir
        global h_last_time
        global v_last_time

        if bm_count > 0:
            bm_count -= 1
            # Scroll in horizontal direction and turn at ends.
            if pg_fig.mne.t_start + pg_fig.mne.duration \
                    >= pg_fig.mne.inst.times[-1]:
                hscroll_dir = False
            elif pg_fig.mne.t_start <= 0:
                hscroll_dir = True
            key = 'right' if hscroll_dir else 'left'
            pg_fig._fake_keypress(key)
            # Get time-difference
            now = perf_counter()
            if h_last_time is not None:
                hscroll_diffs.append(now - h_last_time)
            h_last_time = now
        elif bm_count > -bm_limit:
            bm_count -= 1
            # Scroll in vertical direction and turn at ends.
            if pg_fig.mne.ch_start + pg_fig.mne.n_channels \
                    >= len(pg_fig.mne.inst.ch_names):
                vscroll_dir = False
            elif pg_fig.mne.ch_start <= 0:
                vscroll_dir = True
            key = 'down' if vscroll_dir else 'up'
            pg_fig._fake_keypress(key)
            # get time-difference
            now = perf_counter()
            if v_last_time is not None:
                vscroll_diffs.append(now - v_last_time)
            v_last_time = now
        else:
            timer.stop()
            bm_count = copy(bm_limit)
            hscroll_dir = True
            vscroll_dir = True
            h_last_time = None
            v_last_time = None

            h_mean_fps = 1 / np.median(hscroll_diffs)
            v_mean_fps = 1 / np.median(vscroll_diffs)
            print(f'The median FPS for {benchmark_param} is:\n'
                  f'Horizontal = {h_mean_fps:.3f} FPS\n'
                  f'Vertical = {v_mean_fps:.3f} FPS')
            pg_fig.close()

    app = QApplication.instance()
    if app is None:
        app = QApplication(sys.argv)
    fig = raw_orig.plot(duration=5, n_channels=40,
                        show=False, block=False, **benchmark_param)
    timer = QTimer()
    timer.timeout.connect(partial(_ininite_hscroll, fig))
    timer.start(0)

    fig.show()
    with pytest.raises(SystemExit):
        sys.exit(app.exec())



import mne
import os
import numpy as np

from mne.viz._figure import use_browser_backend


def main():
    sample_data_folder = mne.datasets.sample.data_path()
    sample_data_raw_file = os.path.join(sample_data_folder, 'MEG', 'sample',
                                        'sample_audvis_raw.fif')
    raw = mne.io.read_raw(sample_data_raw_file)

    with use_browser_backend('pyqtgraph'):
        onsets = np.arange(2, 8, 2) + raw.first_time
        durations = np.repeat(1, len(onsets))
        descriptions = ['Test1', 'Test2', 'Test3']
        for onset, duration, description in \
                zip(onsets, durations, descriptions):
            raw.annotations.append(onset, duration, description)

        raw.plot(block=True)


if __name__ == '__main__':
    main()



import pytest

from mne.viz import use_browser_backend
from mne.conftest import *  # noqa: F401, F403


@pytest.fixture
def browser_backend(garbage_collect):
    """Parametrizes the name of the browser backend."""
    with use_browser_backend('pyqtgraph') as backend:
        yield backend



import pathlib

from setuptools import setup

readme = (pathlib.Path(__file__).parent / "README.md").read_text()

setup(name='mne-qt-browser',
      version='0.1.1',
      maintainer='Martin Schulz',
      maintainer_email='dev@earthman-music.de',
      description='A new backend based on pyqtgraph for the 2D-Data-Browser '
                  'in MNE-Python.',
      long_description=readme,
      long_description_content_type='text/markdown',
      license='License :: OSI Approved :: BSD License',
      url='https://github.com/mne-tools/mne-qt-browser',
      download_url='https://github.com/mne-tools/mne-qt-browser/archive/refs'
                   '/tags/v0.1.1.tar.gz',
      project_urls={'Bug Tracker':
                    'https://github.com/mne-tools/mne-qt-browser/issues'},
      classifiers=['Programming Language :: Python :: 3',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent'],
      packages=['mne_qt_browser'],
      install_requires=['numpy',
                        'scipy',
                        'matplotlib',
                        'PyQt5',
                        'qtpy',
                        'mne',
                        'pyqtgraph',
                        'pyopengl'],
      entry_points={'console_scripts':
                       ['mne_qt_browser = mne_qt_browser.__main__:main']}
      )
